#' Analyse the hydraulics of the maquette
#' 
#' @param model either a character, name given to the model or an object of class model
#' @param limnigraph numeric, matrix of floodwater depth heights by instant t for each exterior opening exposed to flooding
#' @param flood_duration numeric, duration of the flood event in seconds, optional; if not provided, calculated using limnigraph
#' @param opening_scenario character, one of the following options: "open", "closed", "combined" or "user_def"
#' @param exchange character, user-defined scenario of exchanges; use in case opening_scenario = "user_def", ignore otherwise
#' @param sim_id character, optional; id of simulation if function called from a loop
#' @param filename character, optional; file name without extension to be given to simulation's output wen saving to disk
#' @param verbose boolean, will floodam tells what it is doing, default to getOption("floodam_building_verbose")
#' 
#' @description {
#' "open" → scenario where all openings are open
#' "closed" → scenario where all openings are closed
#' "combined" → scenario where all exterior openings are closed and all interior openings are open
#' "user_def" ¬ user-defined scenario
#' 
#' When not provided, the flood duration is calculated using the limnigraph as follows: final time in limnigraph + half an hour. 
#' }
#' @return data.frame of hydraulic values: water exchanges through openings and floodwater depth in rooms
#' 
#' @export

analyse_hydraulics = function(
		model,
		limnigraph,
		flood_duration,
		opening_scenario = c("open", "closed", "combined", "user_def"),
		exchange,
		sim_id = NULL,
		filename = NULL,
		verbose = getOption("floodam_building_verbose")
	)
{	
	opening_scenario = match.arg(opening_scenario)
	# initial checks
	if (missing(model)) stop("model not provided")
	if (missing(limnigraph)) stop("limnigraph not provided")
	if (opening_scenario == "user_def" && missing(exchange)) stop("user-defined exchanges not provided") 
	if (missing(flood_duration)) flood_duration = max(limnigraph[["time"]]) + 1800
	if (flood_duration < max(limnigraph[["time"]])) stop("flood duration cannot be shorter than maximum time step in limnigraph")
	if (verbose) cat(sprintf("Simulating hydraulics for '%s'...\n", model[["name"]]))	
	
	# determine opening scenario
	exchange = switch(
		opening_scenario,
		open = model[["hydraulic"]][["exchanges_open"]],
		closed = model[["hydraulic"]][["exchanges_close"]],
		combined = model[["hydraulic"]][["exchanges_combined"]],
		user_def = exchange
	)

	# simulate maquette's interior hydraulics
	result = sim_hydrau_py(
		dataexch = exchange,
		datarooms = model[["hydraulic"]][["rooms"]],
		databound = limnigraph,
		tmax = flood_duration,
		ID = opening_scenario
	)

    names(result) = c("h", "eQ", "eS")

	# calculating flood velocity
	selection = colnames(
		result[["eQ"]]
	)[!grepl("opening_state", colnames(result[["eQ"]]))]
	
	result[["v"]] = result[["eQ"]][ , selection]/result[["eS"]][ , selection]
	result[["v"]][["time"]] = rownames(result[["v"]])
	result[["v"]][["opening_state"]] = opening_scenario

	# adding simulation information
	result = sapply(
		names(result),
		function(x){
			result[[x]][["flood_duration"]] = flood_duration
			result[[x]][["maquette"]] = model[["name"]]
			if(!is.null(sim_id)) result[[x]][["sim_id"]] = sim_id
			return(result[[x]])
		},
		simplify = FALSE
	)

	if (verbose) cat(sprintf("\t... hydraulics successfully modeled for '%s'\n", model[["name"]]))

	if (!is.null(filename)) {
		if (verbose) cat("\t... saving output for flood depth (h) and flow volume (eQ), section (eS) and velocity (v) in: \n")
		lapply(
			names(result),
			function(x) {
				if (verbose) cat(sprintf("\t -%s_%s.csv \n", filename, x))
				utils::write.csv2(result[[x]], sprintf("%s_%s.csv", filename, x))
			}
		)
		if (verbose) cat(sprintf("\t... hydraulic simulation outputs successfully saved in %s_*.csv \n", filename))
	}
		
	return(result)
}

#' Analyse the hydraulics of the maquette
#' 
#' @param dataexch numeric, matrix of openings and their dimensions
#' @param datarooms numeric, matrix of room dimensions
#' @param databound numeric, limnigraph describing the flood event
#' @param dtmax numeric, simulation time step, default to 0.5 (default value fixed by Pascal Finaud-Guyot. Non-modifiable without adjusting python model)
#' @param t numeric, time step where the flood event starts, default to 0
#' @param tmax numeric, time step where the flood event ends
#' @param ID character, simulation ID
#' 
#' @description {
#' The function searches for the python module that calculates the indoor hydraulic behavior, passes values to the hydraulic model and returns results
#' }
#' @return data.frame of hydraulic values: water exchanges through openings and floodwater depth in rooms

sim_hydrau_py = function(
	dataexch,
	datarooms,
	databound,
	dtmax = 0.5,	# Set by Pascal Finaud-Guyot. Non-modifiable without adjusting python model
	t = 0,			# simulation begins always in t = 0
	tmax,
	ID
) 
{
	# load python script
	hydraulic_module = reticulate::import_from_path(
		module = "hydraulic_module",
		path = system.file(
			"python", 
			package = "floodam.building"
		),
		delay_load = TRUE
	)
	# set python version to use
	reticulate::use_python(Sys.which("python3"))
	
	# call to python function and result retrieving
	result = hydraulic_module$calc_hydrau(
		dataexch, 
		datarooms, 
		databound, 
		dtmax, 
		t, 
		tmax, 
		ID
	)
	
	return(result)
}
