#' Analyse a model.
#' 
#' @param model either a character, name given to the model or an object of class model
#' @param type character, type of model that is loaded, default to ""
#' @param stage character, what are the stages that should be done, default to nothing.
#' @param hazard_range if damaging are calculated, gives the range for hazard, default to getOption("floodam_building_hazard_range")
#' @param path list of useful paths, default to getOption("floodam_building_path")
#' @param file_name, list of useful file names, default to getOption("floodam_building_file_name")
#' @param type_building character, type of building file (building or yaml)
#' @param version_building character, version of building model to be used
#' @param verbose boolean, will floodam tells what it is doing, default to getOption("floodam_building_verbose")
#' @param dam_by_room logical. Should the damage functions be computed by room? Default to FALSE
#' @return an object of class model
#' @examples
#' \dontrun{
#' load_model("test", "dwelling")
#' }
#' 
#' @export

analyse_model = function(
		model,
		type = "",
		stage = "", 
		hazard_range = getOption("floodam_building_hazard_range"), 
		path = getOption("floodam_building_path"),
		file_name = getOption("floodam_building_file_name"),
		type_building = getOption("floodam_building_type_building"),
		version_building = NULL,
		verbose = getOption("floodam_building_verbose"),
		dam_by_room = FALSE
	)
{		
	start = Sys.time()
	if (identical(stage, "all")) stage = getOption("floodam_building_stage")
	stage = intersect(as.character(unlist(stage)), getOption("floodam_building_stage"))

    if (length(stage) == 0) {
        if (verbose) cat("No admissible stage asked. Nothing is done.\n\n")
        return(NULL)
    }

	# Load model
	if ("load" %in% stage) {
		model = load_model(
			model_name = model,
			type = type,
			path = path,
			file_name = file_name,
			type_building = type_building,
			version_building = version_building,
			verbose = verbose
		)
	}

    if (! methods::is(model, "model")) {
       if (verbose) cat("At this stage 'model' should be a model but is not. Should you add 'load' in stage? Nothing is done.\n\n")
       return(NULL)
    }

	model_name = model[["name"]]
	file_log = file.path(model[["path"]][["model_log"]], model[["file_name"]][["log"]])

	# Extracting information from building and furniture
	if ("extract" %in% stage) {
		model[["data_table"]] = extract_building_information(
			model,
			verbose
		)
		model[["value"]] = extract_value(
			room = model[["data_table"]][["room"]],
			furniture = model[["data_table"]][["furniture"]],
			category = model[["category"]],
			model_name = model_name,
			verbose = verbose
		)
	}

	# Calculate damaging
	if ("damaging" %in% stage) {
		model[["damaging"]] = compute_damage(
			model,
			hazard_range = hazard_range,
			verbose = verbose,
			dam_by_room = dam_by_room
		)
	}

	# generate input data tables for hydraulic model
	if ("hydraulic" %in% stage) {
		model[["hydraulic"]] = extract_hydraulic_input(
			model,
			verbose = verbose
		)
	}

	# Plotting graph damaging
	if ("graph" %in% stage) {
		if (verbose) cat(sprintf("Plotting graphs for '%s'...\n", model_name))
		selected_view = c("top", "front")
		if (!is.null(model[["damaging"]])) selected_view = c(selected_view, "damaging")
		msg_plot = plot(model, view = selected_view, device = "pdf")
		if (verbose) cat(paste("\t-", msg_plot, "\n"))
	}

	# Saving model
	if ("save" %in% stage) {
		save_model(model, verbose)
	}

	# Plotting something in display
	if ("display" %in% stage) {
		if (verbose) cat(sprintf("Display top view of '%s'\n", model_name))
		plot(model, view = "top", device = "display", finition = "check")
	}

	duration = Sys.time() - start

	if (verbose) {
		cat(
			sprintf(
				"End of analysis for '%s'. Total elapsed time %.2f %s\nMore information availabe at %s\n\n",
				model_name,
				duration,
				attr(duration, "units"),
				file_log
			)
		)
	}
		
	invisible(model)
}

