#' Extract hydraulic input data from model
#' 
#' @param model either a character, name given to the model or an object of class model
#' @param initial_water_depth_interior numeric, initial water depth in each room in meters, default to getOption("floodam_building_hydraulic_initial_water_depth_interior")
#' @param discharge_coefficient numeric, discharge coefficient for each opening, default to getOption("floodam_building_hydraulic_discharge_coefficient")
#' @param closed_opening_margin numeric, measurement of the opening gap clearance, default to getOption("floodam_building_hydraulic_closed_opening_clearance")
#' @param verbose boolean, will floodam tells what it is doing, default to getOption("floodam_building_verbose")
#' @return component hydraulic of the model
#' 
#' @export

extract_hydraulic_input = function(
		model,
		initial_water_depth_interior = getOption("floodam_building_hydraulic_initial_water_depth_interior"),
		discharge_coefficient =	getOption("floodam_building_hydraulic_discharge_coefficient"),
	 	closed_opening_margin = getOption("floodam_building_hydraulic_closed_opening_clearance"),
		verbose = getOption("floodam_building_verbose")
	)
{	
	if (verbose) cat(sprintf("Extracting input data for hydraulic model for '%s'...\n", model[["name"]]))	
	
	opening = unique(model[["data_table"]][["opening"]][["name"]])
	if(any(is.na(opening))) {
		 if (verbose) cat("\t... Openings not named. Cannot determine hydraulic model inputs.\n\n")
        return(NULL)
		}
	
	model[["hydraulic"]] = list()
	# determining exchanges for openings (open & closed)
	exchange_opening = lapply(
		opening, 
		find_exchange, 
		x = model[["data_table"]][["opening"]]
	)
	exchange_opening_open = Reduce(rbind, exchange_opening)
	exchange_opening_open[["discharge_coeff"]] = discharge_coefficient
	exchange_opening_closed = exchange_opening_open
	exchange_opening_closed[["height"]] = closed_opening_margin
	exchange_opening_combined = exchange_opening_open
	exchange_opening_combined[
		grepl("^door|^window", exchange_opening_combined[["exchange"]]), 
		"height"
	] = closed_opening_margin
	
	# determining exchanges for walls
	if("empty" %in% model[["data_table"]][["wall"]][["type"]]) {
		 if (verbose) cat("\t... There are empty walls in your maquette. Determining exchanges.\n\n")
        empty_wall = unique(stats::na.omit(model[["data_table"]][["wall"]][["name"]]))
		if(any(is.na(empty_wall))) {
			if (verbose) cat("\t... Empty walls not named. Cannot determine hydraulic model inputs.\n\n")
			return(NULL)
			}
		exchange_wall = lapply(
			empty_wall, 
			find_exchange, 
			x = model[["data_table"]][["wall"]]
		)
		# calculating median coordinate of empty wall (for dangerousness analysis) and removing duplicated exchanges 
		exchange_wall = lapply(
			exchange_wall,
			function(x) {
				exchange = x[duplicated(x[ ,!colnames(x) %in% c("x","y")]), !colnames(x) %in% c("x","y")]
				med_coords = x[ ,colnames(x) %in% c("x","y")]
				med_coords = apply(med_coords, 2, stats::median)
				return(cbind(exchange,t(med_coords)))
			}
		)
		exchange_wall = Reduce(rbind, exchange_wall)
		# need to convert wall width to cm. Not consistent units across tables (tables mix m and cm) 
		exchange_wall[["width"]] = exchange_wall[["width"]] * 100 
		exchange_wall[["discharge_coeff"]] = discharge_coefficient
		
		# storing exchanges
		exchange_opening_open = rbind(exchange_opening_open, exchange_wall)
		exchange_opening_closed = rbind(exchange_opening_closed, exchange_wall)
		exchange_opening_combined = rbind(exchange_opening_combined, exchange_wall)
	}
	
	# storing exchanges
	model[["hydraulic"]][["exchanges_open"]] = exchange_opening_open
	model[["hydraulic"]][["exchanges_close"]] = exchange_opening_closed
	model[["hydraulic"]][["exchanges_combined"]] = exchange_opening_combined

	# building table of surfaces and heights of rooms
	model[["hydraulic"]][["rooms"]] = model[["data_table"]][["room"]][
    	model[["data_table"]][["room"]][["room"]]!="external_wall", 
    	c("room", "surface", "H_abs")
    ]
	model[["hydraulic"]][["rooms"]][["initial_depth"]] = initial_water_depth_interior

	# converting heights and widths to meters. Necessary for hydraulic model 
	if (verbose) cat(sprintf("\t... converting hydraulic input data in '%s' to meters\n", model[["name"]]))
	
	selection_exchange = c("width", "H_abs", "height")
	selection_room = c("H_abs")

	model[["hydraulic"]] = sapply(
		names(model[["hydraulic"]]),
		function(x) {
			temp = model[["hydraulic"]][[x]]
			if(x == "rooms") {
				temp[ , selection_room] = temp[ , selection_room]/100
			} else {
				temp[ , selection_exchange] = temp[ , selection_exchange]/100
			}
			return(temp)
		},
		simplify = FALSE
	)

	if (verbose) cat(sprintf("\t... hydraulic input data in '%s' succesfully converted to meters\n", model[["name"]]))

	if (verbose) cat(sprintf("\t... hydraulic input data successfully extracted for '%s'\n", model[["name"]]))

	invisible(model[["hydraulic"]])
}

find_exchange = function(x, opening) {
	selection = (x[["name"]] == opening)
	selection[is.na(selection)]=FALSE
    x = x[selection, ]
    exchange = paste0(x[["room"]],collapse = "|")
    if("external_wall" %in% x[["room"]]) {
        temp = x[["room"]][x[["room"]] != "external_wall"]
        exchange = sprintf("%s|%s", unique(x[["name"]]), temp)
    }
    selection = unique(x[,c("width", "H_abs", "height", "x", "y")])
    result = data.frame(exchange, selection)
    return(result)
}
