#' build limnigraph for interior hydraulic simulation
#' 
#' @param model either a character, name given to the model or an object of class model
#' @param time numeric, vector of time steps (in seconds) at which a specific floodwater depth is reached
#' @param depth numeric, vector of floodwater depth (in meters) for each time step
#' @param exposed_openings character, vector of names of openings to the outside exposed to floods in model
#' @param verbose boolean, will floodam tells what it is doing, default to getOption("floodam_building_verbose")
#' @return matrix containing the limnigraph 
#' 
#' @export

generate_limnigraph = function(
		model,
		time,
		depth,
	 	exposed_openings,
        verbose = getOption("floodam_building_verbose") 
	) {	
	if (verbose) cat(sprintf("generating limnigraph for model '%s'...\n", model[["name"]]))	
	if(length(time) != length(depth)) stop("Time and depth do not have the same lengths")
    openings_exterior = as.character(
        model[["data_table"]][["opening"]][
            model[["data_table"]][["opening"]][["room"]] == "external_wall", 
            "name"
        ]
    )
    if(
        !all(exposed_openings %in% openings_exterior)
    ) stop("Please check your openings. I cannot find all of your openings in the openings to the outside of this model")
	
    n_openings = length(openings_exterior)
    n_exposed = length(exposed_openings)

    exposed = matrix(rep(depth, n_exposed), ncol = n_exposed)
    colnames(exposed) = exposed_openings
    exposed = cbind(time, exposed)
    
    not_exposed = matrix(
        rep(0, length(time) * (n_openings-n_exposed)), 
        ncol = n_openings-n_exposed
    )
    colnames(not_exposed) = openings_exterior[
        !openings_exterior %in% exposed_openings
    ]
    not_exposed = cbind(time, not_exposed)


    limnigraph = merge(exposed, not_exposed)

    cat(sprintf("limnigraph for model '%s' successfully generated\n", model[["name"]]))	

	return(limnigraph)
}