#' Inspect building part of model.
#' 
#' @param model model
#' @param pattern list, against which integrity of building information is tested
#' @param verbose boolean, should floodam explains what it is doing
#' @return NULL. A message is provided and a report is written.

inspect_building = function(
        model,
        pattern,
        verbose = getOption("floodam_building_verbose")
    )
{
    if (missing(pattern)) {
        pattern = floodam.building::BUILDING[[getOption("floodam_building_version_building")]]
    }

    extract_element = function(x, status = "ALL"){
        x = x[sapply(x, is.list)]
        if (status == "MANDATORY") x = x[sapply(x, function(x){x[["STATUS"]] == status})]
        
        # General behaviour
        result = list(unique = names(x))
        
        # Dealing with element that may have several occurences
        several = unlist(sapply(x[result[["unique"]]], function(x){if (is.null(x[["SEVERAL"]])) FALSE else x[["SEVERAL"]]}))
        result[["several"]] = result[["unique"]][several]
        result[["unique"]] = result[["unique"]][!several]
        
        # Dealing with element that may have different names (they all may have several occurences)
        result[["generic"]] = switch(status,
                "MANDATORY" = x[["generic"]][[status]],
                unlist(c(x[["generic"]][c("MANDATORY", "OPTIONAL")]), use.names = FALSE))
            if (is.null(result[["generic"]])) result[["generic"]] = character()
        result[["unique"]] = result[["unique"]][result[["unique"]] != "generic"]
        result[["several"]] = result[["several"]][result[["several"]] != "generic"]
        
            return(result)
        }
        
    check_missing = function(n, p) {
            list(
                unique = if (length(p[["unique"]]) > 0) p[["unique"]][!(p[["unique"]] %in% n)] else p[["unique"]],
                several = if (length(p[["several"]]) > 0) p[["several"]][sapply(p[["several"]], function(x) {length(grep(sprintf("^%s", x), n)) == 0})] else p[["several"]],
                generic = if (length(p[["generic"]]) > 0) p[["generic"]][sapply(p[["generic"]], function(x) {length(grep(sprintf("^%s", x), n)) == 0})] else p[["generic"]])
        }
        
        check_present = function(n, p) {
            result = rep(NA, length(n))
            if (length(p[["unique"]]) > 0) result[n %in% p[["unique"]]] = n[n %in% p[["unique"]]]
            for (e in p[["several"]]) {
                selection = grep(e, n)
                if (length(selection) > 0) result[selection] = e
            }
            if (length(p[["generic"]]) > 0) {
                result[sapply(n, function(x) {any(!is.na(pmatch(p[["generic"]], x)))})] = "generic"
            }
            return(result)
        }

    construct_report = function(x, pattern, level = ""){
        report = c()

            # Managing missing elements
            missing = check_missing(names(x), extract_element(pattern, status = "MANDATORY"))
            report = update_report(
        missing[["unique"]],
        sprintf("Each of following elements are missing in %s :", level),
        report
        )
            report = update_report(
        c(missing[["several"]], missing[["generic"]]),
        sprintf("At least one element of each following type is missing in %s :", level),
        report
        )
            
            # Managing present elements, verification if any element is not expected
        present = check_present(names(x), extract_element(pattern))
        report = update_report(
        names(x)[is.na(present)],
        sprintf("Following elements are not expected in %s :", level),
        report
        )
        
        # Managing present elements
        x = x[!is.na(present)]
        present = present[!is.na(present)]
        
        # Verification of the type
        check_type = sapply(x, typeof) == sapply(pattern[present], function(x){x[["VALUE"]]})
        report = update_report(
        names(x)[!check_type],
        sprintf("Trouble with type in %s for the following elements :", level),
        report
        )
        
        # Applying recursive inspection if needed
        recursive = sapply(x, typeof) == "list"
        if (any(recursive)) {
        report = 
            c(
            report,
            unlist(mapply(construct_report, x[recursive], pattern[present[recursive]], paste(level, names(x)[recursive], sep="/")))
            )
        }
        
    report
    }
    
    report = construct_report(model[["building"]], pattern, model[["model_name"]])
    
    if (length(report) > 0) {
        write(
        report,
        file = file.path(
            model[["path"]][["model_log"]],
            model[["file_name"]][["log"]]
        ),
        append = TRUE
        )
        if (verbose) cat(paste("\t- ", report, "\n", sep = ""))
        stop(
        sprintf(
            "%s has some troubles. Look at %s",
            model[["file_name"]][["building"]],
            file.path(model[["path"]][["model_log"]], model[["file_name"]][["log"]])
        )
        )
    } else {
        report = sprintf(
        "\t- Structure of %s of '%s' has been successfully checked\n",
        model[["file_name"]][["building"]],
        model[["name"]]
        )
        write(
        report, 
        file = file.path(
            model[["path"]][["model_log"]],
            model[["file_name"]][["log"]]
        ),
        append = TRUE
        )

        if (verbose) cat(report)
    }
    
    return(NULL)
}


